"""
Application init
"""

# Standard Library
from importlib import metadata

__version__ = metadata.version(distribution_name="tnnt-discordbot-cogs")
__title__ = "TN-NT Discordbot Cogs"

del metadata
